using System;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.CustomFilter
{
    [Example("Custom filters", "Features")]
    public partial class FilterDemoForm : UserControl
    {
        #region Fields

        private readonly Filter _filter;
        private readonly Random _random = new Random();
        
        int? _matchValue;

        #endregion

        public FilterDemoForm()
        {
            //Initialize the grid
            InitializeComponent();

            //Populate the grid
            for (int i = 0; i < 100; ++i)
            {
                RootDataObject rootObject = new RootDataObject(string.Format("Node {0}: To expand the node double click on the row.", grid.Nodes.Count));
                Row root = grid.Rows.Add(rootObject);
                for (int j = 0; j < 10; ++j)
                {
                    root.Add(new DataObject(_random));
                }
            }

            grid.Rows.ExpandAll();
            lbTotalRows.Text = grid.Rows.Count.ToString();
            _filter = new Filter(IsFiltered);
            grid.Filter = _filter;

            //Set initial filter value
            tbFilter.Text = "3";

            
        }

        private void OnPaintCell(object sender, PaintCellEventArgs e)
        {
            if (e.Cell.Row != null && 
                e.Cell.Column != null && 
                e.Cell.Row.Level == 1 &&
                _matchValue != null && 
                IsMatched(e.Cell))
            {
                //Highlight the cell if it matches to the filter's value
                e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, Color.FromArgb(100, Color.Navy));
                e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, Color.FromArgb(100, Color.Navy));
            }
        }

        private void OnExpandAllRows(object sender, EventArgs e)
        {
            grid.Rows.ExpandAll();
        }

        private void OnCollapseAllRows(object sender, EventArgs e)
        {
            grid.Rows.CollapseAll();
        }

        private bool IsFiltered(Row row)
        {
            bool filtered;
            if (row.Level == 0) //This is a root row
            {
                if (!cbEnableEmptyRootRows.Checked)
                {
                    filtered = true;

                    //Hide the root row if all children are filtered
                    foreach (Row child in row.Children)
                    {
                        filtered = IsChildFiltered(child);
                        //The child is not filtered. So, the root row in not filtered too
                        if (!filtered)
                        {
                            break;
                        }
                    }
                }
                else
                {
                    //The root rows should be always visible
                    filtered = false;
                }
            }
            else
            {
                //This is a child row
                filtered = IsChildFiltered(row);
            }
            return filtered;
        }

        private bool IsChildFiltered(Row row)
        {
            bool filtered = false;


            //The grid calls this callback in the GUI thread each time when the value of the data object in changed
            foreach (Column column in grid.Headers[1].VisibleColumns)
            {
                //Get cell only of a child row
                Cell cell = row.Level == 1 ? row[column.Id] : null;
                filtered = !IsMatched(cell);

                //The row can be shown - do not verify remaining columns
                if (!filtered)
                {
                    break;
                }
            }

            //The row is not filtered
            return filtered;
        }

        bool IsMatched(Cell cell)
        {
            bool matched = true;
            if (cell != null && _matchValue != null)
            {
                matched = Equals(_matchValue.Value, cell.Value);
            }
            return matched;
        }

        private void OnFilterUpdated(object sender, EventArgs e)
        {
            //Get a value to match rows
            _matchValue = null;
            int v;
            if (!string.IsNullOrEmpty(tbFilter.Text) &&
                int.TryParse(tbFilter.Text.Trim(), NumberStyles.Number | NumberStyles.AllowThousands | NumberStyles.AllowLeadingWhite, CultureInfo.CurrentCulture, out v))
            {
                _matchValue = v;
            }


            //Notify the Grid about the filter contitions changings
            _filter.NotifyUpdate();
            lbVisibleRows.Text = grid.Rows.Count.ToString();
        }

        private void OnEnableHeadersChanged(object sender, EventArgs e)
        {
            grid.Headers[1].Visible = cbEnableHeaders.Checked;
        }

        private void OnFilterRootRows(object sender, EventArgs e)
        {
            _filter.NotifyUpdate();
            lbVisibleRows.Text = grid.Rows.Count.ToString();
        }
    }
}