using System;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Miscellaneous.Logger
{
    [Example("Logger", "Miscellaneous")]
    public partial class LoggerDemoForm : UserControl
    {
        readonly Timer _timer = new Timer();
        readonly Random _random = new Random();
        private int _messageId;

        public LoggerDemoForm()
        {
            InitializeComponent();

            grid.Filter = new Filter(IsFiltered);

            //Set sort direction
            rbDescendingSort.Checked = true;
            cbToLastInserted.Checked = false;

            //Initialize the timer to add the LogMessages to the grid
            _timer.Interval = 500;
            _timer.Tick += OnTick;
            _timer.Start();
        }

        private void OnPaintGroupRow(object sender, PaintGroupRowEventArgs e)
        {
            int count = e.Row.Children.Count;
            int visibleCount = e.Row.VisibleChildren.Count;
            e.Text += count != visibleCount ? string.Format(", Total instruments: {0}/{1}", visibleCount, count) : string.Format(", Total instruments: {0}", count);

            if (e.Value is Severity)
            {
                e.Image = GetImage((Severity) e.Value);
            }
        }

        private void OnTick(object sender, EventArgs e)
        {
            //Add a new LogMessage to the grid
            Severity severity = (Severity) (_random.Next()%5);
            string message = string.Format("Message id = {0}, Severity = {1}", _messageId++, severity);

            Row row = grid.Rows.Add(new LogMessage(severity, message));
            if(cbToLastInserted.Checked)
            {
                row.EnsureVisible();
            }

            for (int i = 0; i < row.DataAccessor.Count; ++i)
            {
                row[i].Highlight(new TimeSpan(0, 0, 0, 2), Color.FromArgb(180, Color.Red));
            }
            if(row.Parent != null)
            {
                row.Parent.Invalidate();
            }

        }

        private void OnPaintCell(object sender, PaintCellEventArgs e)
        {
            //Override the painting to draw the icons
            if (e.Cell.Row != null && e.Cell.Column != null && e.Cell.Column.Id == "Severity")
            {
                e.Image = GetImage((Severity)e.Cell.Value);

                //Do not paint the text in the "Severity" column
                e.Text = string.Empty;
                e.ImageSettings.Alignment = ContentAlignment.MiddleCenter;
            }
        }

        private void OnUpdateFilter(object sender, EventArgs e)
        {
            //Reevaluate the filter
            grid.FilterRefresh();
        }

        private void OnSortChanged(object sender, EventArgs e)
        {
            //Sort the grid
            if(rbAscendingSort.Checked)
            {
                grid.Sort.Clear();
                grid.Headers[0]["Timestamp"].SortDirection = SortDirection.Ascending;
            }
            else if(rbDescendingSort.Checked)
            {
                grid.Sort.Clear();
                grid.Headers[0]["Timestamp"].SortDirection = SortDirection.Descending;
            }
        }

        private void OnRowAdded(object sender, GridRowEventArgs e)
        {
            //Show the statistics
            lbTotalRows.Text = e.Grid.Nodes.Count.ToString();
            lbVisibleRows.Text = e.Grid.Rows.Count.ToString();
        }

        private void OnBtnClearClick(object sender, EventArgs e)
        {
            //Clear log messages
            grid.Rows.Clear();
        }


        private bool IsFiltered(Row row)
        {
            //Filter implementation
            Severity severity = (Severity)row["Severity"].Value;
            bool filtered = false;
            switch (severity)
            {
                case Severity.Debug:
                    filtered = !cbDebug.Checked;
                    break;

                case Severity.Info:
                    filtered = !cbInfo.Checked;
                    break;
                case Severity.Warn:
                    filtered = !cbWarn.Checked;
                    break;
                case Severity.Error:
                    filtered = !cbError.Checked;
                    break;
                case Severity.Fatal:
                    filtered = !cbFatal.Checked;
                    break;
            }

            //check whether the message matches to the filter expression
            if (!filtered && !string.IsNullOrEmpty(tbFilter.Text))
            {
                try
                {
                    string message = (string)row["Message"].Value;
                    string pattern = tbFilter.Text;
                    pattern = pattern.Replace("*", ".*");
                    pattern = pattern.Replace("?", ".");

                    filtered = !Regex.IsMatch(message, "^" + pattern, RegexOptions.Singleline | RegexOptions.IgnoreCase);
                }
                catch
                {
                    filtered = false;
                }
            }


            return filtered;
        }

        static Image GetImage(Severity severity)
        {
            switch (severity)
            {
                case Severity.Debug:
                    return Resources.debug;
                    
                case Severity.Info:
                    return Resources.info;
                    
                case Severity.Fatal:
                    return Resources.fatal;
                    
                case Severity.Error:
                    return Resources.error;

                case Severity.Warn:
                    return Resources.warning;
            }

            return null;
        }
    }
}